@@ -274,7 +274,7 @@ void PlasmoidPackageTest::createAndInstallPackage()
     QDir rootDir(m_packageRoot + "/plasmoid_to_package");
     rootDir.removeRecursively();
 
-    QVERIFY(QFile::exists(packagePath));
+    QVERIFY2(QFile::exists(packagePath), qPrintable(packagePath));
 
     KZip package(packagePath);
     QVERIFY(package.open(QIODevice::ReadOnly));
@@ -299,13 +299,41 @@ void PlasmoidPackageTest::createAndInstallPackage()
     QVERIFY(spy.wait(1000));
 
     //is the package instance usable (ie proper path) after the install job has been completed?
-    QCOMPARE(p->path(), QString(m_packageRoot % "/plasmoid_to_package/"));
+    QCOMPARE(p->path(), QString(QDir(m_packageRoot % "/plasmoid_to_package").canonicalPath() + QLatin1Char('/')));
     cleanupPackage(QStringLiteral("plasmoid_to_package"));
 
     //QVERIFY(p->isValid());
     delete p;
 }
 
+void PlasmoidPackageTest::noCrashOnAsyncInstall()
+{
+    createTestPackage(QStringLiteral("plasmoid_to_package"), QStringLiteral("1.1"));
+    const QString packagePath = m_packageRoot + '/' + "testpackage.plasmoid";
+
+    KZip creator(packagePath);
+    QVERIFY(creator.open(QIODevice::WriteOnly));
+    creator.addLocalDirectory(m_packageRoot + '/' + "plasmoid_to_package", QStringLiteral("."));
+    creator.close();
+    QDir rootDir(m_packageRoot + "/plasmoid_to_package");
+    rootDir.removeRecursively();
+
+    KJob *job;
+    //scope the package so it will get deleted before the install operation finishes
+    //package is explicitlyshared internally and designed to be used on the stack
+    //see #370718
+    {
+        KPackage::Package package(new KPackage::PackageStructure(this));
+        job = package.install(packagePath, m_packageRoot);
+    }
+    connect(job, SIGNAL(finished(KJob*)), SLOT(packageInstalled(KJob*)));
+    QSignalSpy spy(job, SIGNAL(finished(KJob*)));
+    QVERIFY(spy.wait(1000));
+
+
+    cleanupPackage(QStringLiteral("plasmoid_to_package"));
+}
+
 void PlasmoidPackageTest::createAndUpdatePackage()
 {
     //does the version number parsing work?
@@ -383,6 +411,16 @@ void PlasmoidPackageTest::createAndUpdatePackage()
     delete p;
 }
 
+void PlasmoidPackageTest::uncompressPackageWithSubFolder()
+{
+    KPackage::PackageStructure *structure = new KPackage::PackageStructure;
+    KPackage::Package package(structure);
+    package.setPath(QFINDTESTDATA("data/customcontent.tar.gz"));
+
+    //if metadata is correctly found, servicetypes should be ("SimpleContent", "CustomContent")
+    QCOMPARE(package.metadata().serviceTypes(), QStringList({"SimpleContent", "CustomContent"}));
+}
+
 void PlasmoidPackageTest::cleanupPackage(const QString &packageName)
 {
     KPackage::Package *p = new KPackage::Package(m_defaultPackageStructure);
@@ -391,11 +429,13 @@ void PlasmoidPackageTest::cleanupPackage(const QString &packageName)
 
     QSignalSpy spy(jj, &KJob::finished);
     QVERIFY(spy.wait(1000));
+
 }
 
+
 void PlasmoidPackageTest::packageInstalled(KJob *j)
 {
-    qDebug() << "!!!!!!!!!!!!!!!!!!!! package installed" << (j->error() == KJob::NoError);
+    qDebug() << "!!!!!!!!!!!!!!!!!!!! package installed" << (j->error() == KJob::NoError) << j->errorText();
     QVERIFY(j->error() == KJob::NoError);
 }
 
